// PICSLabelOptions.java
// by Yang-hua Chu
// last update: 10/11/96

package w3c.pics.label;

import java.io.* ;
import java.net.*;
import w3c.pics.util.*;


public class PICSLabelOptions {
  private final int _UNINITIALIZED = -1;
  private final int _TRUE = 1;
  private final int _FALSE = 0;

  private String atOption = null;
  private String byOption = null;
  private String commentOption = null;
  private String expOption = null;     // same as 'until'
  private SExp extensionOption = null;
  private URL forOption = null;
  private URL fullOption = null;    // same as 'comlete-label'
  private int genOption = _UNINITIALIZED;       // same as 'generic'
  private String onOption = null;
  private String md5Option = null;
  // private String RSAOption;

  public boolean isFor(URL inURL, boolean allowGeneric)
  {
    if (forOption == null)   // if 'for' not specified, assume implicit
      return true;
    if (genOption == this._TRUE)
      if (allowGeneric)  
	return inURL.equals(forOption);  // need exact match
      else
	return false;
    return inURL.toString().startsWith(forOption.toString());
  }

  public PICSLabelOptions dup()
  {
    PICSLabelOptions dup = new PICSLabelOptions();
    dup.atOption = atOption;
    dup.byOption = byOption;
    dup.commentOption = commentOption;
    dup.expOption = expOption;
    if (extensionOption != null)
      dup.extensionOption = (SExp) extensionOption.clone();
    try {
    if (forOption != null)
      dup.forOption = new URL(forOption.toString());
    if (fullOption != null)
      dup.fullOption = new URL(fullOption.toString());
    } catch (MalformedURLException e) { }  // should not happen anyway
    dup.genOption = genOption;
    dup.md5Option = md5Option;
    dup.onOption = onOption;
    return dup;
  }

  // return a union of 'this' and 'subOpts', if a conflict arises,
  // 'this' takes precedence.
  // leave both 'this' and 'subOpts' unchanged
  public PICSLabelOptions merge(PICSLabelOptions subOpts)
  {
    PICSLabelOptions a = this.dup();
    PICSLabelOptions b = subOpts.dup();

    if (a.atOption == null)      //string
      a.atOption = b.atOption;  
    if (a.byOption == null)     //string
      a.byOption = b.byOption;  
    if (a.commentOption == null)      //string
      a.commentOption = b.commentOption;
    if (a.expOption == null)     //string
      a.expOption = b.expOption;
    if (a.extensionOption == null)     // SExp
      a.extensionOption = b.extensionOption;
    if (a.fullOption == null)  //URL
      a.fullOption = b.fullOption;
    if (a.forOption == null)  //URL
      a.forOption = b.forOption;
    if (a.genOption == _UNINITIALIZED)  //int
      a.genOption = b.genOption;
    if (a.md5Option == null)   // string
      a.md5Option = b.md5Option;
    if (a.onOption == null)   // string
      a.onOption = b.onOption;
    return a;
  }

  public String getOptionBy ()
  {
    return byOption;
  }

  public void getOptionBy (String s)
  {
    byOption = s;
  }


  public boolean parse(SExp exp)
    throws PICSParseException
  {
    boolean success = false;
    
    while (true) {
      if (! exp.expect(exp.SEXP_IDENT)) {
	exp.pushBack();
	break;
      }
      String opt = exp.current().getIdent();
      // currently I am allowing overriding of option

      // 'at' quoted-ISO-date
      if (opt.equals("at")) {
	if (! exp.expect(exp.SEXP_STRING)) 
	  throw new PICSParseException(exp.errorMsg("quotename for at"));
	  atOption = exp.current().getString();
	  success = true;
	  continue;
      } 

      // 'by' quotename
      if (opt.equals("by")) {
	if (exp.expect(exp.SEXP_STRING)) {
	  byOption = exp.current().getString();
	  success = true;
	  continue;
	} else {
	  throw new PICSParseException(exp.errorMsg("quotename for by"));
	}
      }

      // 'comment' quotename
      if (opt.equals("comment")) {
	if (! exp.expect(exp.SEXP_STRING))
	  throw new PICSParseException
	    (exp.errorMsg("a quotename followed by 'comment' option"));
	commentOption = exp.current().getString();
	success = true;
	continue;
      }

      // 'exp' | 'until' quoted-ISO-date
      if (opt.equals("exp") || opt.equals("until")) {
	if (! exp.expect(exp.SEXP_STRING))
	  throw new PICSParseException
	    (exp.errorMsg("a quoted-ISO-date followed 'exp' or 'until'"));
	expOption = exp.current().getString();
	success = true;
	continue;
      }

      // option :: 'extension' '(' mand/opt quotedURL data* ')'
      // mand/opt :: 'mandatory' | 'option' 
      if (opt.equals("extension")) {
	if (! exp.expect(exp.SEXP_SEXP))
	  throw new PICSParseException
	    (exp.errorMsg("a s-expression followed by extension option"));
	extensionOption = exp.current();
	success = true;
	continue;
      }

      // 'for' quoteURL
      if (opt.equals("for")) {
	if (! exp.expect(exp.SEXP_STRING))
	  throw new PICSParseException
	    (exp.errorMsg("a String followed by 'for' option"));
	try { forOption = new URL(exp.current().getString());  }
	catch (MalformedURLException e) {
	  throw new PICSParseException(exp.errorMsg("bad URL")); }
	success = true;
	continue;
      }
      
      // 'full' | 'complete-label'  quoteURL
      if (opt.equals("full") || opt.equals("complete-label")) {
	if (! exp.expect(exp.SEXP_STRING))
	  throw new PICSParseException 
	    (exp.errorMsg("an URL followed by " + opt + " option"));
	try { fullOption = new URL(exp.current().getString());  }
	catch (MalformedURLException e) {
	  throw new PICSParseException(exp.errorMsg("bad URL after " +opt)); }
	success = true;
	continue;
      }
      
      // 'generic' | 'gen' boolean
      if (opt.equals("gen") || opt.equals("generic")) {
	if (exp.expect(exp.SEXP_IDENT, "true")) {
	  genOption = this._TRUE;
	  success = true;
	  continue;
	}
	if (exp.expect(exp.SEXP_IDENT, "false")) {
	  genOption = this._FALSE;
	  success = true;
	  continue;
	}
	throw new PICSParseException
	  (exp.errorMsg("true or false follow generic"));
      }
      
      // 'md5' base64
      if (opt.equals("md5")) {
	if (! exp.expect(exp.SEXP_STRING))
	  throw new PICSParseException
	    (exp.errorMsg("a base64 followed by 'md5' option"));
	md5Option = exp.current().getString();
	success = true;
	continue;
      }

      // 'on' quoted-ISO-date
      if (opt.equals("on")) {
	if (! exp.expect(exp.SEXP_STRING))
	  throw new PICSParseException
	    (exp.errorMsg("a string followed by 'on' option"));
	onOption = exp.current().getString();
	success = true;
	continue;
      }

      exp.pushBack();
      break;
    }
    return success;
  }
  

  // return the label-option key and label-option value
  public String toString()
  {
    String s = "";
    if (atOption != null)
      s += "at " + "\"" + atOption + "\"";
    if (byOption != null)
      s += "by " + "\"" + byOption + "\"";
    if (commentOption != null)
      s += "comment " + "\"" + commentOption + "\"";
    if (expOption != null)
      s += "exp " + "\"" + expOption + "\"";
    if (extensionOption != null)
      s += "extension " + extensionOption.toString();
    if (forOption != null)
      s += "for " + "\"" + forOption.toString() + "\"";
    if (fullOption != null)
      s += "full " + "\"" + fullOption.toString() + "\"";
    if (genOption == this._TRUE)
      s += "generic true";
    if (genOption == this._FALSE)
      s += "generic false";
    if (md5Option != null)
      s += "md5 " + "\"" + md5Option.toString() + "\"";
    if (onOption != null)
      s += "on " + "\"" + onOption.toString() + "\"";
    return s;
  }
}






